@@ -24,7 +24,7 @@ from .exceptions import (
 logger = logging.getLogger('xnat-utils')
 
 
-conv_choices = [f.lower() for f in resource_exts if f != 'DICOM']
+conv_choices = ['nifti', 'nifti_gz', 'mrtrix', 'mrtrix_gz']
 converter_choices = ('dcm2niix', 'mrconvert')
 
 
@@ -436,6 +436,8 @@ def _download_resource(resource_name, download_dir, session_label,
             if (resource_name == 'DICOM' and convert_to in ('nifti',
                                                             'nifti_gz')):
                 msg = 'either dcm2niix or '
+            else:
+                msg = ''
             raise XnatUtilsUsageError(
                 "Please install {} mrconvert to convert between {}"
                 "and {} formats".format(
